unit mwActEvents;
{--------------------------------------------------------------------------+
 | unit mwActEvents
 |
 | Author:  Martin_Waldenburg
 | Created: 8.98
 | Version: 1.1
 | Copyright (c) 1998 Martin Waldenburg
 | All rights reserved.
 |
 | LICENCE CONDITIONS
 |
 | USE OF THE ENCLOSED SOFTWARE
 | INDICATES YOUR ASSENT TO THE
 | FOLLOWING LICENCE CONDITIONS.
 |
 |
 |
 | These Licence Conditions are exlusively
 | governed by the Law and Rules of the
 | Federal Republic of Germany.
 |
 | Redistribution and use in source and binary form, with or without
 | modification, are permitted provided that the following conditions
 | are met:
 |
 | 1. Redistributions of source code must retain the above copyright
 |    notice, this list of conditions and the following disclaimer.
 |    If the source is modified, the complete original and unmodified
 |    source code has to distributed with the modified version.
 |
 | 2. Redistributions in binary form must reproduce the above
 |    copyright notice, these licence conditions and the disclaimer
 |    found at the end of this licence agreement in the documentation
 |    and/or other materials provided with the distribution.
 |
 | 3. Software using this code must contain a visible line of credit.
 |
 | 4. If my code is used in a "for profit" product, you have to donate
 |    to a registered charity in an amount that you feel is fair.
 |    You may use it in as many of your products as you like.
 |    Proof of this donation must be provided to the author of 
 |    this software.
 |
 | 5. If you for some reasons don't want to give public credit to the
 |    author, you have to donate three times the price of your software
 |    product, or any other product including this component in any way,
 |    but no more than $500 US and not less than $200 US, or the
 |    equivalent thereof in other currency, to a registered charity.
 |    You have to do this for every of your products, which uses this
 |    code separately.
 |    Proof of this donations must be provided to the author of 
 |    this software.
 |
 |
 | DISCLAIMER:
 |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR 'AS IS'.
 | 
 | ALL EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 | PARTICULAR PURPOSE ARE DISCLAIMED.
 |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 | (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 | OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 | INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 | WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 | NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 |
 |  Martin.Waldenburg@T-Online.de
+--------------------------------------------------------------------------+}

interface

uses SysUtils,
  Classes,
  Forms,
  Windows,
  Controls,
  TypInfo,
  ActnList,
  Consts;

type
  TmActionEvents = class(TObject)
  private
    Class procedure ActListsChange(Sender: TObject);
    Class procedure ActListsExecute(Action: TBasicAction;
      var Handled: Boolean);
    Class procedure ActListsUpdate(Action: TBasicAction;
      var Handled: Boolean);
    Class procedure ScanForActLists(aComponent: TComponent);
    Class procedure AssignListHandlers;
    Class procedure FreeListHandlers;
    Class procedure TriggerNotifyEvent(Const anEvent; Sender: TObject);
    Class procedure TriggerActionEvent(Const anEvent; Action: TBasicAction; var Handled: Boolean);
  protected
  public
    Class procedure Actualize;
    Class procedure AddListHandler(Const aChange, aExecute, aUpdate);
    Class procedure RemoveListHandler(Const aChange, aExecute, aUpdate);
  end;

var
  ListOfActLists: TList;
  OnChangeList: TList;
  OnExecuteList: TList;
  OnUpdateList: TList;

implementation

Class procedure TmActionEvents.AddListHandler(Const aChange, aExecute, aUpdate);
begin
  if Pointer(aChange) <> nil then if OnChangeList.IndexOf(Pointer(aChange)) < 0 then OnChangeList.Add(Pointer(aChange));
  if Pointer(aExecute) <> nil then if OnExecuteList.IndexOf(Pointer(aExecute)) < 0 then OnExecuteList.Add(Pointer(aExecute));
  if Pointer(aUpdate) <> nil then if OnUpdateList.IndexOf(Pointer(aUpdate)) < 0 then OnUpdateList.Add(Pointer(aUpdate));
end;

Class procedure TmActionEvents.RemoveListHandler(Const aChange, aExecute, aUpdate);
begin
  if Pointer(aChange) <> nil then OnChangeList.Remove(Pointer(aChange));
  if Pointer(aExecute) <> nil then OnExecuteList.Remove(Pointer(aExecute));
  if Pointer(aUpdate) <> nil then OnUpdateList.Remove(Pointer(aUpdate));
end;

Class procedure TmActionEvents.ScanForActLists(aComponent: TComponent);
var
  I: Integer;
begin
  if (PTypeInfo(aComponent.ClassParent.ClassInfo)^.Name = 'TCustomActionList')
   or (PTypeInfo(aComponent.ClassParent.ClassInfo)^.Name = 'TActionList') then
   begin
     ListOfActLists.Add(aComponent);
   end;
  for I:= 0 to aComponent.ComponentCount -1 do
  begin
   ScanForActLists(aComponent.Components[I]);
  end;
  aComponent:= nil;
  AssignListHandlers;
end;

Class procedure TmActionEvents.AssignListHandlers;
var
  I: Integer;
begin
  // because the event handlers of all action lists
  // in the IDE are nil I can do a simple assignment.
  for I:= 0 to ListOfActLists.Count -1 do
  begin
    TActionList(ListOfActLists[I]).OnChange:= ActListsChange;
    TActionList(ListOfActLists[I]).OnExecute:= ActListsExecute;
    TActionList(ListOfActLists[I]).OnUpdate:= ActListsUpdate;
  end;
end;

Class procedure TmActionEvents.FreeListHandlers;
var
  I: Integer;
begin
  for I:= 0 to ListOfActLists.Count -1 do
  begin
    TActionList(ListOfActLists[I]).OnChange:= nil;
    TActionList(ListOfActLists[I]).OnExecute:= nil;
    TActionList(ListOfActLists[I]).OnUpdate:= nil;
  end;
  ListOfActLists.Clear;
end;

Class procedure TmActionEvents.Actualize;
begin
  FreeListHandlers;
  ScanForActLists(Application);
end;

Class procedure TmActionEvents.TriggerActionEvent(const anEvent;
  Action: TBasicAction; var Handled: Boolean);
begin
  TActionEvent(anEvent)(Action, Handled);
end;

Class procedure TmActionEvents.TriggerNotifyEvent(const anEvent; Sender: TObject);
begin
  TNotifyEvent(anEvent)(Sender);
end;

Class procedure TmActionEvents.ActListsChange(Sender: TObject);
var
  I: Integer;
  aPointer: Pointer;
begin
  for I:= 0 to OnChangeList.Count -1 do
  begin
    aPointer:= OnChangeList[I];
    TriggerNotifyEvent(aPointer, Sender);
    aPointer:= nil;
  end;
end;

Class procedure TmActionEvents.ActListsExecute(Action: TBasicAction;
  var Handled: Boolean);
var
  I: Integer;
  aPointer: Pointer;
begin
  for I:= 0 to OnExecuteList.Count -1 do
  begin
    aPointer:= OnExecuteList[I];
    TriggerActionEvent(aPointer, Action, Handled);
    aPointer:= nil;
  end;
end;

Class procedure TmActionEvents.ActListsUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  I: Integer;
  aPointer: Pointer;
begin
  for I:= 0 to OnUpdateList.Count -1 do
  begin
    aPointer:= OnUpdateList[I];
    TriggerActionEvent(aPointer, Action, Handled);
    aPointer:= nil;
  end;
end;

initialization
  ListOfActLists:= TList.Create;
  OnChangeList:= TList.Create;
  OnExecuteList:= TList.Create;
  OnUpdateList:= TList.Create;
  TmActionEvents.Actualize;

finalization
  TmActionEvents.FreeListHandlers;
  ListOfActLists.Free;
  OnChangeList.Free;
  OnExecuteList.Free;
  OnUpdateList.Free;

end.
